@@ -18,13 +18,16 @@ class Colour():
     def __getitem__(self, index):
         return [self.r, self.g, self.b][index]
 
+    def __repr__(self):
+        return f'Colour{self.r, self.g, self.b}'
+
 
 class Palette():
     def __init__(self, palette_file=None):
         self.transparent = None
         self.entries = []
 
-        if type(palette_file) is Palette:
+        if isinstance(palette_file, Palette):
             self.transparent = palette_file.transparent
             self.entries = palette_file.entries
             return
@@ -107,6 +110,21 @@ class Palette():
 
         self.image = palette.convert('RGBA')
 
+    def quantize_image(self, image, transparent=None, strict=False):
+        if strict and len(self) == 0:
+            raise TypeError("Attempting to enforce strict colours with an empty palette, did you really want to do this?")
+        w, h = image.size
+        output_image = Image.new('P', (w, h))
+        for y in range(h):
+            for x in range(w):
+                r, g, b, a = image.getpixel((x, y))
+                if transparent is not None and (r, g, b) == tuple(transparent):
+                    a = 0x00
+                index = self.get_entry(r, g, b, a, strict=strict)
+                output_image.putpixel((x, y), index)
+
+        return output_image
+
     def get_entry(self, r, g, b, a, remap_transparent=True, strict=False):
         if (r, g, b, a) in self.entries:
             index = self.entries.index((r, g, b, a))
@@ -128,49 +146,14 @@ class Palette():
         else:
             raise TypeError(f'Colour {r}, {g}, {b}, {a} does not exist in palette!')
 
-    def tolist(self):
-        result = []
-        for r, g, b, a in self.entries:
-            result.append(r)
-            result.append(g)
-            result.append(b)
-            result.append(a)
-        return result
-
-    def tobytes(self):
-        return bytes(self.tolist())
-
     def tostruct(self):
-        result = []
-        for r, g, b, a in self.entries:
-            result.append({
-                'r': r,
-                'g': g,
-                'b': b,
-                'a': a,
-            })
-        return result
+        return [dict(zip('rgba', c)) for c in self.entries]
 
-    def bit_length(self):
-        return max(1, len(self.entries) - 1).bit_length()
+    def __iter__(self):
+        return iter(self.entries)
 
     def __len__(self):
         return len(self.entries)
 
     def __getitem__(self, index):
         return self.entries[index]
-
-
-def type_palette(palette_file):
-    # Only used as a type in argparse.
-    # This wrapper around Palette traps errors and
-    # raises in a way that's visible to the user
-    if type(palette_file) is Palette:
-        return palette_file
-
-    try:
-        return Palette(palette_file)
-    except TypeError as e:
-        raise argparse.ArgumentTypeError(None, str(e))
-    except ValueError as e:
-        raise argparse.ArgumentError(None, str(e))
